#!/usr/bin/env python

import skimage.restoration
from skimage import img_as_float
import numpy as np
from PIL import Image
import os
from tqdm import tqdm
import csv
import rawpy

imagesFolderPath = 'flat-field/nef'
imagesFolderPathFileName = imagesFolderPath.replace('/', '_')
denoiser = 'wavelet'
npArrayFilePath = f'mean_{imagesFolderPathFileName}_{denoiser}.npy'

denoise = getattr(skimage.restoration, f'denoise_{denoiser}')

mean = None
numberOfImagesInMean = 0

imagesFileNames = os.listdir(imagesFolderPath)

requiresRaiseFiltering = False
if requiresRaiseFiltering:
    files = {}

    with open('RAISE_all.csv') as csvfile:
        reader = csv.DictReader(csvfile)
        for row in tqdm(list(reader), 'CSV parsing'):
            file = row['File'] + '.png'
            files[file] = row

    imagesFileNames = [imageFileName for imageFileName in tqdm(imagesFileNames, 'Filtering images') if files[imageFileName]['Device'] == 'Nikon D7000' and Image.open(f'{imagesFolderPath}/{imageFileName}').size == (4946, 3278)]

def treatImage(imageFileName):
    global mean, numberOfImagesInMean
    imageFilePath = f'{imagesFolderPath}/{imageFileName}'
    if imageFileName.endswith('.NEF'):
        imageFileName = 'r01b6aed6t.NEF'
        imageFilePath = f'{imagesFolderPath}/{imageFileName}'
        with rawpy.imread(imageFilePath) as raw:
            colorDesc = raw.color_desc.decode('ascii')
            assert colorDesc == 'RGBG'
            rawImageVisible = raw.raw_image_visible.copy()
            centerY = 628
            centerX = 3_014
            RANGE = 5
            rawImageVisible = rawImageVisible.astype(np.float64)
            rawImageVisibleShape = rawImageVisible.shape
            for y in range(0, rawImageVisibleShape[0], 2):
                for x in range(0, rawImageVisibleShape[1], 2):
                    rawImageVisible[y, x] *= -1
            print(rawImageVisible[centerY - RANGE:centerY + RANGE, centerX - RANGE:centerX + RANGE])
            #greenRawImageVisible = raw.rawImageVisible
            # Pay attention to range of values expected by the denoiser.
            exit(1)
    else:
        imagePil = Image.open(imageFilePath)
        imageNpArray = img_as_float(np.array(imagePil))
    # As the arguments differ from a denoiser to the other should use a match instead.
    imageDenoisedNpArray = denoise(imageNpArray, channel_axis=-1, convert2ycbcr=True, rescale_sigma=True)
    imageNoiseNpArray = imageNpArray - imageDenoisedNpArray
    if mean is None:
        mean = imageNoiseNpArray
    else:
        mean = ((mean * numberOfImagesInMean) + imageNoiseNpArray) / (numberOfImagesInMean + 1)
    numberOfImagesInMean += 1

for imageFileName in tqdm(imagesFileNames, 'Denoising images'):
    treatImage(imageFileName)

with open(npArrayFilePath, 'wb') as f:
    np.save(f, mean)
